//*** The code for the simulations of the article 
//*** "Mathematical modelling shows that the response of a solid tumor to antiangiogenic therapy depends on the type of its growth"
//*** by Maxim Kuznetsov
//*** journal "Mathematics", Special Issue "Mathematical Modelling in Biomedicine"
//*** 2020
//*** Version 2 -- invasive tumor, only intergrowth is considered (subsection 3.2)
//*** This code works fine in Dev-C++, version 5.5.2 
//*** Note, that some small parts of the code may need to be revised depending on the utilized software 

//***********************************************************************************************************************************************************
//********************************* INTRODUCTION ************************************************************************************************************
//***********************************************************************************************************************************************************

//********* Initializing needful things

#include <stdio.h>
#include <tchar.h>
#include <windows.h>
#include <ctime>
#include <iostream>
#include <cmath>
#include <conio.h>
#include <fstream>

using namespace std;

//********* MODEL PARAMETERS

double B = 0.01; // proliferation rate of tumor cells 
double Q = 12.; // rate of glucose consumption by tumor cells
double Dg = 100.; // glucose diffusion coefficient
double P = 4.; // permeability of capillaries to glucose
double gcr = 0.1; // critical level of glucose for cells
double Dn = 0.1; // intrinsic motility of tumor cells
double M = 0.05; // death rate of tumor cells
double R = 0.2; // rate of degradation of capillaries
double eps = 100.; // sensitivity of tumor cell death rate to glucose level

//********* SIMULATION PARAMETERS

#define size_cm 5. // the length of a simulation region, in cm
#define tau 0.01 // time step
#define h 0.1 // space step
#define N (int(100.*size_cm/h)) // number of points in space, minus one
#define output_frequency 24. // how often the distributions of varibles will be recorded
#define rsn_frequency 0.1 // how often the tumor radius, speed of tumor front and number of tumor cells will be recorded
FILE *results;
#define inception_width_mm 0.01 // the width of inital tumor cell population, in mm
#define inception_width_int (int(10.*inception_width_mm / h)) // the number of points in space of inital tumor cell population, in cm

//********* COUNTERS

double t = 0.; // main
double ti = 0.; // for output of the distributions of varibles
double tr = 0.; // for output of the tumor radius, speed of tumor front and number of tumor cells
int d = 0; // for serial numbers of the files with distributions of varibles

//********* MODEL VARIABLES

double n0[N + 1], nr[N + 1], n1[N + 1]; // tumor cells
double m0[N + 1], m1[N + 1]; // necrosis
double g0[N + 1], gr[N + 1], g1[N + 1]; // glucose
double c0[N + 1], c1[N + 1]; // capillaries

//********* PROCEDURES AND FUNCTIONS

void Begin(); // complex of actions for the beginning, ...
void Cycle_Ending(); // ...ending of main cycle, ...
void End(); // ...and ending of the program excution

void Migration_cells(); //  solving the equation for tumor cells migration
void Diffusion_glucose(); //  solving the equation for diffusion of glucose
void TMA(double apr, double bpr, double cpr, double F[N + 1], double prog[N + 1],
	double muleft, double hileft, double muright, double hiright); // tridiagonal matrix algorithm with constant b
void Euler(); // Euler method
double Growthn(double nn, double mg); // kinetic terms for tumor cells
double Growthm(double nn, double mg); // kinetic terms for necrosis
double Growthg(double nn, double gg, double mg, double cc); // kinetic terms for glucose
double Growthc(double nn, double mm, double cc); // kinetic terms for capillaries
double sigma(double gg); // function that defines the behavior of tumor cells depending on the level of glucose
#define Nm 10000
double Nmd = (0.5 / Nm);
double Mm[Nm + 1]; // values for sigma(g) are precalculated for optimization purposes
double Mg;
void Profile_output(); // output of distributions of variables
void Radius_Speed_CellNumber_output(); // output of the tumor radius, speed of tumor front and number of tumor cells
void Radius_estimation(); // estimation of tumor radius
void Speed_estimation(); // estimation of tumor front speed

//********* AUXILIARY VARIABLES

////// for tumor cells migration

double kn = (tau*Dn) / (h*h);

////// for glucose diffusion

double kg = 2 * h*h / (tau*Dg);
double aprg = 1.;
double bprg = 2. + kg;
double cprg = 1.;
double Fg[N + 1];
double muleftg, hileftg, murightg, hirightg;
double progg[N + 1];
double pp[N + 1], qq[N + 1];

////// for tumor radius, speed of tumor front and number of tumor cells

double nR = 0.1; // the "detectable" amount of tumor cells
double Rad; // current tumor radius, in mm
double Rad_prev; // previously measured tumor radius, in mm
double T_prev; // time of previous measurment of tumor radius
double V; // tumor front speed
double NTC; // relative number of tumor cells

////// for work with files for distribution of variables

char File_name1[80], num[10];

//***********************************************************************************************************************************************************
//********************************* MAIN CYCLE **************************************************************************************************************
//***********************************************************************************************************************************************************

int main()
{
	Begin();

	// the program runs by default  until a preassigned time or until tumor approaches the right boundary or until the capillaries density on the right boundary sags
	while ((n0[N - 1] < 0.00001) && (c0[N - 1] > 0.99999))
	{
		t = t + tau;
		Radius_estimation();
		Speed_estimation();
		
		if (t >= ti - 0.00001) // a trifle is subtracted to overcome the inevitable minor machine errors during arithmetic operations
			Profile_output();
		if (t >= tr)
			Radius_Speed_CellNumber_output();

		Euler();
		Migration_cells();
		Diffusion_glucose();

		Cycle_Ending();
	}

	End();
}

//***********************************************************************************************************************************************************
//********************************* BEGINNING OF SIMULATION *************************************************************************************************
//***********************************************************************************************************************************************************

void Begin()
{
	//********* PRECALCULATING sigma(g)
	for (int i = 0; i <= Nm; i++)
		Mm[i] = sigma(1.*i / Nm);

	//********* CREATING FILE FOR TUMOR RADIUS, FRONT SPEED AND CELL NUMBER
	if ((results = fopen("results.txt", "wt")) == NULL)
		fprintf(stderr, "Cannot open file results.txt\n\n");

	fprintf(results, "time(h)\trad(mm)\tspeed\tcells\n\n");
	fflush(results);

	//********* INITIAL CONDITIONS

	for (int i = 0; i <= N; i++)
	{
		n0[i] = 0.;
		m0[i] = 0.;
		g0[i] = 1.;
		c0[i] = 1.;
	}

	for (int i = 0; i <= inception_width_int; i++)
	{
		n0[i] = 1.;
		c0[i] = 0.;
	}

	for (int i = 0; i <= N; i++) // this is made for easy output of the initial conditions
	{
		n1[i] = n0[i];
		m1[i] = m0[i];
		g1[i] = g0[i];
		c1[i] = c0[i];
	}
}

//***********************************************************************************************************************************************************
//********************************* OUTPUT OF DISTRIBUTIONS OF VARIABLES ************************************************************************************
//***********************************************************************************************************************************************************

void Profile_output()
{
	//********* CREATING A FILENAME WITH SERIAL NUMBER

	d = (int)((ti + 0.00001) / output_frequency);
	_itoa(d, num, 10);

	if (d < 10)
		strcpy(File_name1, "res-d000"); // designed for maximum a thousand of files
	else if (d < 100)
		strcpy(File_name1, "res-d00");
	else if (d  < 1000)
		strcpy(File_name1, "res-d0");
	else
		strcpy(File_name1, "res-d");

	strcat(File_name1, num);
	strcat(File_name1, ".txt");

	//********* PRINTING THE SERIAL NUMBER, RADIUS, SPEED OF TUMOR FRONT AND NUMBER OF TUMOR CELLS ON A SCREEN

	printf("day %i, radius = %.3f mm, speed = %.3f mm/week, cell number = %.3f\n", d, Rad, V, NTC);

	//********* OUTPUT OF DISTRIBUTIONS OF VARIABLES INTO A FILE

	FILE * out;
	if ((out = fopen(File_name1, "wt")) == NULL)
		fprintf(stderr, "Cannot open file\n");

	fprintf(out, "Len(mm) n               m               g               c\n\n");

	for (int i = 0; i <= N; i++)
		fprintf(out, "%.3f\t%.7e\t%.7e\t%.7e\t%.7e\n", i*h / 10., n1[i], m1[i], g1[i], c1[i]);
	fclose(out);

	//********* SWITCHING THE COUNTER FOR OUTPUT OF DISTRIBUTION OF VARIABLES 

	ti = ti + output_frequency;
}

//***********************************************************************************************************************************************************
//********************************* ESTIMATION OF RADIUS AND SPEED ******************************************************************************************
//***********************************************************************************************************************************************************

//********* ESTIMATION OF RADIUS

void Radius_estimation()
{
	Rad = 0.;

	for (int i = 0; i <= N - 1; i++)
		if ((n1[i] >= nR) && (n1[i + 1]<nR))
			Rad = i*h / 10.;
}

//********* ESTIMATION OF TUMOR FRONT SPEED

void Speed_estimation()
{
	if (abs(Rad - Rad_prev)>0.1 && Rad > inception_width_mm + 0.1)
	{
		V = 7.*24.*(Rad - Rad_prev) / (t - T_prev); // in mm/week
		T_prev = t;
		Rad_prev = Rad;
	}
}

//***********************************************************************************************************************************************************
//********************************* OUTPUT OF THE TUMOR RADIUS, SPEED OF TUMOR FRONT AND NUMBER OF TUMOR CELLS **********************************************
//***********************************************************************************************************************************************************

void Radius_Speed_CellNumber_output()
{
	tr = tr + rsn_frequency;

	//********* COUNTING TUMOR CELLS
	NTC = 0.;

	for (int i = 0; i <= N; i++)
		NTC = NTC + n1[i] * h; // normalized so that there are 10. relative units for 1 mm of length with n=1.

	//********* OUTPUT

	fprintf(results, "%.1f\t%.5f\t%.5f\t%.5f\n", t, Rad, V, NTC);
	fflush(results);
}

//***********************************************************************************************************************************************************
//********************************* SOLUTION OF KINETIC TERMS ******************************************************************************************
//***********************************************************************************************************************************************************

//********* FUNCTION THAT DEFINES THE BEHAVIOR OF TUMOR CELLS DEPENDING ON THE LEVEL OF GLUCOSE

double sigma(double gg)
{
	return (0.5*(1. - tanh(eps*(gg - gcr))));
}

//********* KINETIC FUNCTIONS

double Growthn(double nn, double mg) // for tumor cells
{
	return(B*nn*(1. - mg) - M*nn*mg);
}

double Growthm(double nn, double mg) // for necrosis
{
	return (M*nn*mg);
}

double Growthg(double nn, double gg, double mg, double cc) // for glucose
{
	return (P*cc*(1. - gg) - Q*nn*(1. - mg));
}

double Growthc(double nn, double mm, double cc) // for capillaries
{
	return (-R*(nn + mm)*cc);
}

//********* EULER PROCEDURE

void Euler()
{
	for (int i = 0; i <= N; i++)
	{
		Mg = Mm[(int)((g0[i] + Nmd) *Nm)];

		nr[i] = n0[i] + tau*Growthn(n0[i], Mg);
		m1[i] = m0[i] + tau*Growthm(n0[i], Mg);
		gr[i] = g0[i] + tau*Growthg(n0[i], g0[i], Mg, c0[i]);
		c1[i] = c0[i] + tau*Growthc(n0[i], m0[i], c0[i]);
	}
}

//***********************************************************************************************************************************************************
//********************************* MIGRATION OF CELLS AND DIFFUSION OF GLUCOSE *****************************************************************************
//***********************************************************************************************************************************************************

//********* PROCEDURE FOR CELL MIGRATION 

void Migration_cells()
{
	for (int i = 1; i <= N - 1; i++)
		n1[i] = kn*(n0[i - 1] - 2.*n0[i] + n0[i + 1]) + nr[i];

	n1[0] = 2.*kn*(n0[1] - n0[0]) + nr[0];
	n1[N] = 2.*kn*(n0[N - 1] - n0[N]) + nr[N];
}

//********* PROCEDURE FOR GLUCOSE DIFFUSION

void Diffusion_glucose()
{
	for (int i = 1; i <= N - 1; i++)
		Fg[i] = (g0[i + 1] - 2.*g0[i] + g0[i - 1]) + kg * gr[i];

	muleftg = (kg * gr[0] + 2.*(g0[1] - g0[0])) / (kg + 2.);
	hileftg = 2. / (kg + 2.);

	murightg = (kg * gr[N] + 2.*(g0[N - 1] - g0[N])) / (kg + 2.);
	hirightg = 2. / (kg + 2.);

	TMA(aprg, bprg, cprg, Fg, progg, muleftg, hileftg, murightg, hirightg);

	for (int i = 0; i <= N; i++)
		g1[i] = progg[i];
}

//********* TRIDIAGONAL MATRIX ALGORITHM

void TMA(double apr, double bpr, double cpr, double* F, double* prog,
	double muleft, double hileft, double muright, double hiright)
{
	pp[1] = hileft;
	qq[1] = muleft;

	for (int i = 1; i <= N - 1; i++)
	{
		pp[i + 1] = cpr / (bpr - pp[i] * apr);
		qq[i + 1] = (apr*qq[i] + F[i]) / (bpr - pp[i] * apr);
	}

	prog[N] = (muright + hiright*qq[N]) / (1. - pp[N] * hiright);

	for (int i = N - 1; i >= 0; i--)
		prog[i] = pp[i + 1] * prog[i + 1] + qq[i + 1];
}

//***********************************************************************************************************************************************************
//********************************* MAIN CYCLE ENDING *******************************************************************************************************
//***********************************************************************************************************************************************************

void Cycle_Ending()
{
	for (int i = 0; i <= N; i++)
	{
		g0[i] = g1[i];
		n0[i] = n1[i];
		m0[i] = m1[i];
		c0[i] = c1[i];
	}
}

//***********************************************************************************************************************************************************
//********************************* PROGRAM ENDING **********************************************************************************************************
//***********************************************************************************************************************************************************

void End()
{
	if (clock() / CLOCKS_PER_SEC >= 3600)
		printf("\n%s%ld%s%ld%s%ld%s", "The end. Total time: ", (clock() / CLOCKS_PER_SEC) / 3600, " h ", (clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600), " min ", (clock() / CLOCKS_PER_SEC) - 3600 * ((clock() / CLOCKS_PER_SEC) / 3600) - 60 * ((clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600)), " s.");
	else if (clock() / CLOCKS_PER_SEC >= 60)
		printf("\n%s%ld%s%ld%s", "The end. Total time: ", (clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600), " min ", (clock() / CLOCKS_PER_SEC) - 3600 * ((clock() / CLOCKS_PER_SEC) / 3600) - 60 * ((clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600)), " s.");
	else
		printf("\n%s%ld%s", "The end. Total time: ", (clock() / CLOCKS_PER_SEC) - 3600 * ((clock() / CLOCKS_PER_SEC) / 3600) - 60 * ((clock() / CLOCKS_PER_SEC) / 60 - 60 * ((clock() / CLOCKS_PER_SEC) / 3600)), " s.");
	_getch();
}

//********************************************************
//********* THE END
//********************************************************
